package de.kif.backend

import de.kif.backend.database.Connection
import de.kif.backend.model.User
import de.kif.backend.route.*
import io.ktor.application.Application
import io.ktor.application.ApplicationCall
import io.ktor.application.install
import io.ktor.application.log
import io.ktor.auth.Authentication
import io.ktor.auth.FormAuthChallenge
import io.ktor.auth.UserPasswordCredential
import io.ktor.auth.form
import io.ktor.features.*
import io.ktor.http.content.files
import io.ktor.http.content.static
import io.ktor.locations.Location
import io.ktor.locations.Locations
import io.ktor.response.respondRedirect
import io.ktor.routing.routing
import io.ktor.sessions.*
import io.ktor.util.hex
import io.ktor.websocket.WebSockets
import kotlinx.coroutines.launch
import kotlinx.serialization.json.Json

data class PortalSession(val id: Int, val name: String) {
    suspend fun getUser(call: ApplicationCall): User {
        val user = User.find(name)
        if (user == null || user.id != id) {
            call.sessions.clear<PortalSession>()
            call.respondRedirect("/login?error")
            throw IllegalAccessException()
        }
        return user
    }
}

@Location("/")
class LocationDashboard()

@Location("/calendar/{day}")
data class LocationCalendar(val day: Int) {
    @Location("/{room}/{time}")
    data class LocationCalendarEdit(
        val calendar: LocationCalendar,
        val room: Int,
        val time: Int,
        val search: String = ""
    ) {
        val day: Int get() = calendar.day
    }

    @Location("/{room}/{time}/{workGroup}")
    data class LocationCalendarSet(
        val calendar: LocationCalendar,
        val room: Int,
        val time: Int,
        val workGroup: Int,
        val next: String? = null
    ) {
        val day: Int get() = calendar.day
    }

    @Location("/time-to-room")
    data class LocationCalendarTimeToRoom(val calendar: LocationCalendar){
        val day: Int get() = calendar.day
    }

    @Location("/room-to-time")
    data class LocationCalendarRoomToTime(val calendar: LocationCalendar){
        val day: Int get() = calendar.day
    }
}

@Location("/login")
data class LocationLogin(val username: String = "", val password: String = "", val next: String = "/")

@Location("/logout")
class LocationLogout()

@Location("/account")
class LocationAccount()

@Location("/user")
data class LocationUser(val search: String = "") {
    @Location("/{id}")
    data class Edit(val id: Int)

    @Location("/new")
    class New()

    @Location("/{id}/delete")
    data class Delete(val id: Int)
}

@Location("/workgroup")
data class LocationWorkGroup(val search: String = "") {
    @Location("/{id}")
    data class Edit(val id: Int)

    @Location("/new")
    class New()

    @Location("/{id}/delete")
    data class Delete(val id: Int)
}

@Location("/track")
data class LocationTrack(val search: String = "") {
    @Location("/{id}")
    data class Edit(val id: Int)

    @Location("/new")
    class New()

    @Location("/{id}/delete")
    data class Delete(val id: Int)
}

@Location("/room")
data class LocationRoom(val search: String = "") {
    @Location("/{id}")
    data class Edit(val id: Int)

    @Location("/new")
    class New()

    @Location("/{id}/delete")
    data class Delete(val id: Int)
}

@Location("/person")
data class LocationPerson(val search: String = "") {
    @Location("/{id}")
    data class Edit(val id: Int)

    @Location("/new")
    class New()

    @Location("/{id}/delete")
    data class Delete(val id: Int)
}

fun Application.main() {
    Connection.init()

    install(DefaultHeaders)
    install(CallLogging)
    install(ConditionalHeaders)
    install(Compression)
    install(DataConversion)
    install(Locations)
    install(WebSockets)

    install(Authentication) {
        form {
            userParamName = LocationLogin::username.name
            passwordParamName = LocationLogin::password.name
            challenge = FormAuthChallenge.Redirect { _ ->
                "/login?error"
            }
            validate { credential: UserPasswordCredential ->
                val user = User.find(credential.name) ?: return@validate null
                if (user.checkPassword(credential.password)) user else null
            }
        }
    }

    val sessionKey = hex("1234567890abcdef") //TODO
    install(Sessions) {
        cookie<PortalSession>("SESSION") {
            transform(SessionTransportTransformerMessageAuthentication(sessionKey))
        }
    }

    routing {
        static("/static") {
            files(Resources.directory)
        }

        launch {
            val firstStart = User.exists()
            if (firstStart) {
                log.info("Please create the first user and restart the server!")
                setup()
            } else {
                dashboard()
                calendar()
                login()
                account()

                workGroup()
                track()
                room()
                person()
                user()

                pushService()
            }
        }
    }
}