package de.kif.backend.model

import de.kif.backend.database.DbConstraintType
import de.kif.backend.database.DbPersonConstraint
import de.kif.backend.database.dbQuery
import org.jetbrains.exposed.sql.deleteWhere
import org.jetbrains.exposed.sql.insert
import org.jetbrains.exposed.sql.select
import org.jetbrains.exposed.sql.update

sealed class PersonConstraint(
    var id: Int = -1
) {

    abstract suspend fun save(personId: Int)

    suspend fun delete() {
        val id = id
        if (id >= 0) {
            dbQuery {
                DbPersonConstraint.deleteWhere { DbPersonConstraint.id eq id }
            }
        }
    }

    class BeginOnDay(
        var time: Long = 0,
        var day: Int = 0
    ) : PersonConstraint() {
        override suspend fun save(personId: Int) {
            if (id < 0) {
                dbQuery {
                    val newId = DbPersonConstraint.insert {
                        it[this@insert.personId] = personId
                        it[type] = DbConstraintType.BEGIN
                        it[time] = this@BeginOnDay.time
                        it[day] = this@BeginOnDay.day
                    }[DbPersonConstraint.id]!!
                    this@BeginOnDay.id = newId
                }
            } else {
                dbQuery {
                    DbPersonConstraint.update({ DbPersonConstraint.id eq id }) {
                        it[this@update.personId] = personId
                        it[type] = DbConstraintType.BEGIN
                        it[time] = this@BeginOnDay.time
                        it[day] = this@BeginOnDay.day
                    }
                }
            }
        }
    }

    class EndOnDay(
        var time: Long = 0,
        var day: Int = 0
    ) : PersonConstraint() {
        override suspend fun save(personId: Int) {
            if (id < 0) {
                dbQuery {
                    val newId = DbPersonConstraint.insert {
                        it[this@insert.personId] = personId
                        it[type] = DbConstraintType.END
                        it[time] = this@EndOnDay.time
                        it[day] = this@EndOnDay.day
                    }[DbPersonConstraint.id]!!
                    this@EndOnDay.id = newId
                }
            } else {
                dbQuery {
                    DbPersonConstraint.update({ DbPersonConstraint.id eq id }) {
                        it[this@update.personId] = personId
                        it[type] = DbConstraintType.END
                        it[time] = this@EndOnDay.time
                        it[day] = this@EndOnDay.day
                    }
                }
            }
        }
    }

    class BlockedOnDay(
        var time: Long = 0,
        var duration: Int = 0,
        var day: Int = 0
    ) : PersonConstraint() {

        override suspend fun save(personId: Int) {
            if (id < 0) {
                dbQuery {
                    val newId = DbPersonConstraint.insert {
                        it[this@insert.personId] = personId
                        it[type] = DbConstraintType.BLOCKED
                        it[time] = this@BlockedOnDay.time
                        it[duration] = this@BlockedOnDay.duration
                        it[day] = this@BlockedOnDay.day
                    }[DbPersonConstraint.id]!!
                    this@BlockedOnDay.id = newId
                }
            } else {
                dbQuery {
                    DbPersonConstraint.update({ DbPersonConstraint.id eq id }) {
                        it[this@update.personId] = personId
                        it[type] = DbConstraintType.BLOCKED
                        it[time] = this@BlockedOnDay.time
                        it[duration] = this@BlockedOnDay.duration
                        it[day] = this@BlockedOnDay.day
                    }
                }
            }
        }
    }

    companion object {
        suspend fun get(personId: Int): Set<PersonConstraint> = dbQuery {
            val result = DbPersonConstraint.select { DbPersonConstraint.personId eq personId }
            result.map {
                val id = it[DbPersonConstraint.id]
                val type = it[DbPersonConstraint.type]
                val time = it[DbPersonConstraint.time]
                val duration = it[DbPersonConstraint.duration]
                val day = it[DbPersonConstraint.day]

                when (type) {
                    DbConstraintType.BEGIN -> PersonConstraint.BeginOnDay(time, day)
                    DbConstraintType.END -> PersonConstraint.EndOnDay(time, day)
                    DbConstraintType.BLOCKED -> PersonConstraint.BlockedOnDay(time, duration, day)
                }.also { it.id = id }
            }.toSet()
        }
    }
}
