package de.kif.backend.model

import de.kif.backend.database.DbUser
import de.kif.backend.database.DbUserPermission
import de.kif.backend.database.dbQuery
import io.ktor.auth.Principal
import org.jetbrains.exposed.sql.*
import org.mindrot.jbcrypt.BCrypt

class User(
    var id: Int = -1,
    var username: String = "",
    private var password: String = ""
) : Principal {
    var permissions: Set<Permission> = emptySet()

    fun checkPassword(password: String): Boolean {
        return BCrypt.checkpw(password, this.password)
    }

    fun hashPassword(newPassword: String) {
        password = BCrypt.hashpw(newPassword, BCrypt.gensalt())
    }

    fun checkPermission(permission: Permission): Boolean {
        return permission in permissions || Permission.ADMIN in permissions
    }

    suspend fun loadPermissions() = dbQuery {
        permissions = DbUserPermission.slice(DbUserPermission.permission).select {
            DbUserPermission.userId eq id
        }.map { it[DbUserPermission.permission] }.toSet()
    }

    suspend fun save() {
        if (id < 0) {
            dbQuery {
                val newId = DbUser.insert {
                    it[username] = this@User.username
                    it[password] = this@User.password
                }[DbUser.userId]!!
                this@User.id = newId

                for (permission in permissions) {
                    DbUserPermission.insert {
                        it[userId] = newId
                        it[this.permission] = permission
                    }
                }
            }
        } else {
            dbQuery {
                DbUser.update({ DbUser.userId eq id }) {
                    it[username] = this@User.username
                    it[password] = this@User.password
                }

                DbUserPermission.deleteWhere { DbUserPermission.userId eq id }

                for (permission in permissions) {
                    DbUserPermission.insert {
                        it[userId] = id
                        it[this.permission] = permission
                    }
                }
            }
        }
    }

    suspend fun delete() {
        val id = id
        if (id >= 0) {
            dbQuery {
                DbUserPermission.deleteWhere { DbUserPermission.userId eq id }
                DbUser.deleteWhere { DbUser.userId eq id }
            }
        }
    }

    companion object {
        suspend fun create(username: String, password: String, permissions: Set<Permission>) {
            val user = User(username = username)
            user.hashPassword(password)
            user.permissions = permissions
            user.save()
        }

        suspend fun find(username: String): User? = dbQuery {
            val result = DbUser.select { DbUser.username eq username }.firstOrNull() ?: return@dbQuery null
            User(result[DbUser.userId], result[DbUser.username], result[DbUser.password])

        }?.apply { loadPermissions() }

        suspend fun get(userId: Int): User? = dbQuery {
            val result = DbUser.select { DbUser.userId eq userId }.firstOrNull() ?: return@dbQuery null
            User(result[DbUser.userId], result[DbUser.username], result[DbUser.password])
        }?.apply { loadPermissions() }

        suspend fun list(): List<User> = dbQuery {
            val query = DbUser.selectAll()
            query.map { result ->
                User(result[DbUser.userId], result[DbUser.username], result[DbUser.password])
            }
        }.onEach { it.loadPermissions() }

        suspend fun exists(): Boolean = dbQuery {
            DbUser.selectAll().count() == 0
        }
    }
}
