package de.kif.backend.model

import de.kif.backend.database.DbConstraintType
import de.kif.backend.database.DbWorkGroupConstraint
import de.kif.backend.database.dbQuery
import org.jetbrains.exposed.sql.deleteWhere
import org.jetbrains.exposed.sql.insert
import org.jetbrains.exposed.sql.select
import org.jetbrains.exposed.sql.update

sealed class WorkGroupConstraint(
    var id: Int = -1
) {

    abstract suspend fun save(workGroupId: Int)

    suspend fun delete() {
        val id = id
        if (id >= 0) {
            dbQuery {
                DbWorkGroupConstraint.deleteWhere { DbWorkGroupConstraint.id eq id }
            }
        }
    }

    class BeginOnDay(
        var time: Long = 0,
        var day: Int = 0
    ) : WorkGroupConstraint() {
        override suspend fun save(workGroupId: Int) {
            if (id < 0) {
                dbQuery {
                    val newId = DbWorkGroupConstraint.insert {
                        it[this@insert.workGroupId] = workGroupId
                        it[type] = DbConstraintType.BEGIN
                        it[time] = this@BeginOnDay.time
                        it[day] = this@BeginOnDay.day
                    }[DbWorkGroupConstraint.id]!!
                    this@BeginOnDay.id = newId
                }
            } else {
                dbQuery {
                    DbWorkGroupConstraint.update({ DbWorkGroupConstraint.id eq id }) {
                        it[this@update.workGroupId] = workGroupId
                        it[type] = DbConstraintType.BEGIN
                        it[time] = this@BeginOnDay.time
                        it[day] = this@BeginOnDay.day
                    }
                }
            }
        }
    }

    class EndOnDay(
        var time: Long = 0,
        var day: Int = 0
    ) : WorkGroupConstraint() {
        override suspend fun save(workGroupId: Int) {
            if (id < 0) {
                dbQuery {
                    val newId = DbWorkGroupConstraint.insert {
                        it[this@insert.workGroupId] = workGroupId
                        it[type] = DbConstraintType.END
                        it[time] = this@EndOnDay.time
                        it[day] = this@EndOnDay.day
                    }[DbWorkGroupConstraint.id]!!
                    this@EndOnDay.id = newId
                }
            } else {
                dbQuery {
                    DbWorkGroupConstraint.update({ DbWorkGroupConstraint.id eq id }) {
                        it[this@update.workGroupId] = workGroupId
                        it[type] = DbConstraintType.END
                        it[time] = this@EndOnDay.time
                        it[day] = this@EndOnDay.day
                    }
                }
            }
        }
    }

    class BlockedOnDay(
        var time: Long = 0,
        var duration: Int = 0,
        var day: Int = 0
    ) : WorkGroupConstraint() {

        override suspend fun save(workGroupId: Int) {
            if (id < 0) {
                dbQuery {
                    val newId = DbWorkGroupConstraint.insert {
                        it[this@insert.workGroupId] = workGroupId
                        it[type] = DbConstraintType.BLOCKED
                        it[time] = this@BlockedOnDay.time
                        it[duration] = this@BlockedOnDay.duration
                        it[day] = this@BlockedOnDay.day
                    }[DbWorkGroupConstraint.id]!!
                    this@BlockedOnDay.id = newId
                }
            } else {
                dbQuery {
                    DbWorkGroupConstraint.update({ DbWorkGroupConstraint.id eq id }) {
                        it[this@update.workGroupId] = workGroupId
                        it[type] = DbConstraintType.BLOCKED
                        it[time] = this@BlockedOnDay.time
                        it[duration] = this@BlockedOnDay.duration
                        it[day] = this@BlockedOnDay.day
                    }
                }
            }
        }
    }

    companion object {
        suspend fun get(workGroupId: Int): Set<WorkGroupConstraint> = dbQuery {
            val result = DbWorkGroupConstraint.select { DbWorkGroupConstraint.workGroupId eq workGroupId }
            result.map {
                val id = it[DbWorkGroupConstraint.id]
                val type = it[DbWorkGroupConstraint.type]
                val time = it[DbWorkGroupConstraint.time]
                val duration = it[DbWorkGroupConstraint.duration]
                val day = it[DbWorkGroupConstraint.day]

                when (type) {
                    DbConstraintType.BEGIN -> WorkGroupConstraint.BeginOnDay(time, day)
                    DbConstraintType.END -> WorkGroupConstraint.EndOnDay(time, day)
                    DbConstraintType.BLOCKED -> WorkGroupConstraint.BlockedOnDay(time, duration, day)
                }.also { it.id = id }
            }.toSet()
        }
    }
}
