package de.kif.backend.route

import de.kif.backend.LocationCalendar
import de.kif.backend.LocationLogin
import de.kif.backend.PortalSession
import de.kif.backend.model.Permission
import de.kif.backend.model.Room
import de.kif.backend.model.Schedule
import de.kif.backend.model.WorkGroup
import de.kif.backend.util.Search
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.backend.view.TableTemplate
import io.ktor.application.call
import io.ktor.html.insert
import io.ktor.html.respondHtmlTemplate
import io.ktor.locations.get
import io.ktor.request.path
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.routing.get
import io.ktor.sessions.get
import io.ktor.sessions.sessions
import kif.common.model.CALENDAR_GRID_WIDTH
import kif.common.model.MessageCreateCalendarEntry
import kif.common.model.MessageDeleteCalendarEntry
import kotlinx.css.CSSBuilder
import kotlinx.css.Color
import kotlinx.css.pct
import kotlinx.css.rem
import kotlinx.html.*
import kotlin.math.max
import kotlin.math.min

const val MINUTES_OF_DAY = 24 * 60

private fun DIV.calendarCell(schedule: Schedule?, day: Int, time: Int) {
    if (schedule != null) {
        span("calendar-entry") {
            attributes["style"] = CSSBuilder().apply {
                val size = schedule.workGroup.length / CALENDAR_GRID_WIDTH.toDouble()
                val pos = (schedule.time % CALENDAR_GRID_WIDTH) / CALENDAR_GRID_WIDTH.toDouble()

                this.left = (pos * 100).pct
                this.top = (pos * 100).pct + 0.1.rem

                this.width = (size * 100).pct
                this.height = (size * 100).pct - 0.2.rem

                val c = schedule.workGroup.track?.color
                if (c != null) {
                    backgroundColor = Color(c.toString())
                    color = Color(c.textColor.toString())
                }
            }.toString()
            attributes["data-language"] = schedule.workGroup.language.code
            attributes["data-day"] = schedule.day.toString()
            attributes["data-room"] = schedule.room.id.toString()
            attributes["data-time"] = schedule.time.toString()
            attributes["data-cell-time"] = time.toString()
            attributes["data-workgroup"] = schedule.workGroup.id.toString()

            +schedule.workGroup.name

            div("calendar-tools") {
                a(
                    classes = "calendar-tools-m10",
                    href = "/calendar/$day/${schedule.room.id}/${schedule.time}/-1?next=/calendar/$day/${schedule.room.id}/${schedule.time - 10}/${schedule.workGroupId}"
                ) { +"-10" }
                a(
                    classes = "calendar-tools-m5",
                    href = "/calendar/$day/${schedule.room.id}/${schedule.time}/-1?next=/calendar/$day/${schedule.room.id}/${schedule.time - 5}/${schedule.workGroupId}"
                ) { +"-05" }
                a(
                    classes = "calendar-tools-reset",
                    href = "/calendar/$day/${schedule.room.id}/${schedule.time}/-1?next=/calendar/$day/${schedule.room.id}/$time/${schedule.workGroupId}"
                ) { +"reset" }
                a(
                    classes = "calendar-tools-p5",
                    href = "/calendar/$day/${schedule.room.id}/${schedule.time}/-1?next=/calendar/$day/${schedule.room.id}/${schedule.time + 5}/${schedule.workGroupId}"
                ) { +"+05" }
                a(
                    classes = "calendar-tools-p10",
                    href = "/calendar/$day/${schedule.room.id}/${schedule.time}/-1?next=/calendar/$day/${schedule.room.id}/${schedule.time + 10}/${schedule.workGroupId}"
                ) { +"+10" }
                a(
                    classes = "calendar-tools-del",
                    href = "/calendar/$day/${schedule.room.id}/${schedule.time}/-1"
                ) { +"del" }
            }
        }
    }
}

private fun DIV.renderTimeToRoom(
    day: Int,
    from: Int,
    to: Int,
    rooms: List<Room>,
    schedules: Map<Room, Map<Int, Schedule>>,
    allowEdit: Boolean
) {
    val gridLabelWidth = 60
    val minutesOfDay = to - from

    div("calendar-table-time-to-room") {
        div("calendar-header") {
            div("calendar-cell") {
                span {
                    +"Room"
                }
            }

            for (i in 0 until minutesOfDay / gridLabelWidth) {
                div("calendar-cell") {
                    span {
                        val time = ((i * gridLabelWidth + from) % MINUTES_OF_DAY).let {
                            if (it < 0) it + MINUTES_OF_DAY else it
                        }
                        val minutes = (time % 60).toString().padStart(2, '0')
                        val hours = (time / 60).toString().padStart(2, '0')
                        +"$hours:$minutes"
                    }
                }
            }
        }

        for (room in rooms) {
            div("calendar-row") {
                div("calendar-cell") {
                    span {
                        +room.name
                    }
                }

                for (i in 0 until minutesOfDay / CALENDAR_GRID_WIDTH) {
                    val start = i * CALENDAR_GRID_WIDTH + from
                    val end = (i + 1) * CALENDAR_GRID_WIDTH + from - 1

                    div("calendar-cell") {
                        val time = i * CALENDAR_GRID_WIDTH
                        val minutes = (time % 60).toString().padStart(2, '0')
                        val hours = (time / 60).toString().padStart(2, '0')
                        title = "$hours:$minutes"
                        attributes["data-time"] = time.toString()
                        attributes["data-room"] = room.id.toString()
                        attributes["data-day"] = day.toString()

                        val schedule = (start..end).mapNotNull { schedules[room]?.get(it) }.firstOrNull()

                        calendarCell(schedule, day, time)

                        val href = if (allowEdit) "/calendar/$day/${room.id}/$start" else null
                        a(href, classes = "calendar-link")
                    }
                }
            }
        }
    }
}

private fun DIV.renderRoomToTime(
    day: Int,
    from: Int,
    to: Int,
    rooms: List<Room>,
    schedules: Map<Room, Map<Int, Schedule>>,
    allowEdit: Boolean
) {
    val gridLabelWidth = 60
    val minutesOfDay = to - from

    div("calendar-table-room-to-time") {
        div("calendar-header") {
            div("calendar-cell") {
                span {
                    +"Room"
                }
            }

            for (room in rooms) {
                div("calendar-cell") {
                    span {
                        +room.name
                    }
                }
            }
        }

        for (i in 0 until minutesOfDay / CALENDAR_GRID_WIDTH) {
            val time = ((i * CALENDAR_GRID_WIDTH + from) % MINUTES_OF_DAY).let {
                if (it < 0) it + MINUTES_OF_DAY else it
            }
            val minutes = (time % 60).toString().padStart(2, '0')
            val hours = (time / 60).toString().padStart(2, '0')
            val timeString = "$hours:$minutes"

            val start = i * CALENDAR_GRID_WIDTH + from
            val end = (i + 1) * CALENDAR_GRID_WIDTH + from - 1

            div("calendar-row") {
                div("calendar-cell") {
                    if (time % gridLabelWidth == 0) {
                        span {
                            +timeString
                        }
                    }
                }

                for (room in rooms) {
                    div("calendar-cell") {
                        attributes["data-time"] = time.toString()
                        attributes["data-room"] = room.id.toString()
                        attributes["data-day"] = day.toString()
                        title = timeString

                        val schedule = (start..end).mapNotNull { schedules[room]?.get(it) }.firstOrNull()

                        calendarCell(schedule, day, time)

                        val href = if (allowEdit) "/calendar/$day/${room.id}/$start" else null
                        a(href, classes = "calendar-link")
                    }
                }
            }
        }
    }
}

fun Route.calendar() {

    get("/calendar") {
        call.respondRedirect("/calendar/0", true)
    }

    get<LocationCalendar.LocationCalendarRoomToTime> { param ->
        call.response.cookies.append(
            "orientation",
            CalendarOrientation.ROOM_TO_TIME.name,
            maxAge = Int.MAX_VALUE,
            path = "/"
        )
        call.respondRedirect("/calendar/${param.day}")
    }

    get<LocationCalendar.LocationCalendarTimeToRoom> { param ->
        call.response.cookies.append(
            "orientation",
            CalendarOrientation.TIME_TO_ROOM.name,
            maxAge = Int.MAX_VALUE,
            path = "/"
        )
        call.respondRedirect("/calendar/${param.day}")
    }

    get<LocationCalendar> { param ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        val allowEdit = user?.checkPermission(Permission.SCHEDULE) ?: false
        val rooms = Room.list()

        val orientation = call.request.cookies["orientation"]?.let { name ->
            CalendarOrientation.values().find { it.name == name }
        } ?: CalendarOrientation.ROOM_TO_TIME

        val day = param.day
        val h = Schedule.getByDay(day)
        val schedules = h.groupBy { it.room }.mapValues { (_, it) ->
            it.associateBy {
                it.time
            }
        }

        var max = 0
        var min = 24 * 60
        for (s in h) {
            max = max(max, s.time + s.workGroup.length)
            min = min(min, s.time)
        }

        if (min > max) {
            val h1 = max
            max = min
            min = h1
        }

        min = (min / 60 - 1) * 60
        max = (max / 60 + 2) * 60

        min = min(min, 0)

        call.respondHtmlTemplate(MainTemplate()) {
            menuTemplate {
                this.user = user
                active = MenuTemplate.Tab.CALENDAR
            }
            content {
                if (rooms.isEmpty()) {
                    return@content
                }

                div("header") {
                    div("header-left") {
                        a("/calendar/${day - 1}") { +"<" }
                        span {
                            +"Day $day"
                        }
                        a("/calendar/${day + 1}") { +">" }
                    }
                    div("header-right") {
                        a("/calendar/$day/room-to-time") {
                            +"Room to time"
                        }
                        a("/calendar/$day/time-to-room") {
                            +"Time to room"
                        }
                    }
                }

                div("calendar") {
                    attributes["data-day"] = day.toString()

                    div {
                        when (orientation) {
                            CalendarOrientation.ROOM_TO_TIME -> renderRoomToTime(
                                day,
                                min,
                                max,
                                rooms,
                                schedules,
                                allowEdit
                            )
                            CalendarOrientation.TIME_TO_ROOM -> renderTimeToRoom(
                                day,
                                min,
                                max,
                                rooms,
                                schedules,
                                allowEdit
                            )
                        }
                    }
                }
            }

        }
    }

    get<LocationCalendar.LocationCalendarEdit> { param ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.SCHEDULE)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val list = WorkGroup.list()
            val room = Room.get(param.room)
            val day = param.day
            val time = param.time

            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.WORK_GROUP
                }
                content {
                    h1 { +"Select work groups" }
                    insert(TableTemplate()) {
                        searchValue = param.search

                        action {
                            a("/calendar/$day/${room.id}/$time/-1") {
                                button(classes = "form-btn btn-primary") {
                                    +"Delete"
                                }
                            }
                        }

                        header {
                            th {
                                +"Name"
                            }
                            th {
                                +"Interested"
                            }
                            th {
                                +"Track"
                            }
                            th {
                                +"Projector"
                            }
                            th {
                                +"Resolution"
                            }
                            th {
                                +"Length"
                            }
                        }

                        for (u in list) {
                            if (Search.match(param.search, u.name)) {
                                val href = "/calendar/$day/${room.id}/$time/${u.id}"
                                entry {
                                    attributes["data-search"] = Search.pack(u.name)
                                    td {
                                        a(href) {
                                            +u.name
                                        }
                                    }
                                    td {
                                        +u.interested.toString()
                                    }
                                    td {
                                        +(u.track?.name ?: "")
                                    }
                                    td {
                                        +u.projector.toString()
                                    }
                                    td {
                                        +u.resolution.toString()
                                    }
                                    td {
                                        +u.length.toString()
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    get<LocationCalendar.LocationCalendarSet> { param ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.SCHEDULE)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            for (it in Schedule.getByRoom(param.room, param.day, param.time)) {
                PushService.notify(
                    MessageDeleteCalendarEntry(
                        it.day,
                        it.time,
                        it.room.id,
                        it.workGroup.id
                    )
                )
                it.delete()
            }
            if (param.workGroup >= 0) {
                val schedule = Schedule(param.workGroup, param.day, param.time, param.room)
                schedule.save()
                schedule.loadConstraints()

                val cellTime = (schedule.time / 15)  * 15

                PushService.notify(
                    MessageCreateCalendarEntry(
                        schedule.day,
                        schedule.time,
                        cellTime,
                        schedule.room.id,
                        schedule.workGroup.id,
                        schedule.workGroup.name,
                        schedule.workGroup.length,
                        schedule.workGroup.language.code,
                        schedule.workGroup.track?.color
                    )
                )
            }

            call.respondRedirect(param.next ?: "/calendar/${param.day}")
        }
    }
}

enum class CalendarOrientation {
    /**
     * Columns contains time
     * Rows contains rooms
     *
     * Like the old kif tool
     */
    TIME_TO_ROOM,

    /**
     * Columns contains rooms
     * Rows contains time
     *
     * Like the congress schedule
     */
    ROOM_TO_TIME
}
