package de.kif.backend.route

import de.kif.backend.LocationLogin
import de.kif.backend.LocationPerson
import de.kif.backend.PortalSession
import de.kif.backend.model.Permission
import de.kif.backend.model.Person
import de.kif.backend.util.Search
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.backend.view.TableTemplate
import io.ktor.application.call
import io.ktor.html.insert
import io.ktor.html.respondHtmlTemplate
import io.ktor.locations.get
import io.ktor.locations.post
import io.ktor.request.path
import io.ktor.request.receiveParameters
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.sessions.get
import io.ktor.sessions.sessions
import io.ktor.util.toMap
import kotlinx.html.*

fun Route.person() {
    get<LocationPerson> { param ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.PERSON)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val list = Person.list()
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.PERSON
                }
                content {
                    h1 { +"Persons" }
                    insert(TableTemplate()) {
                        searchValue = param.search

                        action {
                            a("/person/new") {
                                button(classes="form-btn btn-primary") {
                                    +"Add person"
                                }
                            }
                        }

                        header {
                            th {
                                +"First name"
                            }
                            th {
                                +"Last name"
                            }
                            th(classes = "action") {
                                +"Action"
                            }
                        }

                        for (u in list) {
                            if (Search.match(param.search, u.firstName, u.lastName)) {
                                entry {
                                    attributes["data-search"] = Search.pack(u.firstName, u.lastName)
                                    td {
                                        +u.firstName
                                    }
                                    td {
                                        +u.lastName
                                    }
                                    td(classes = "action") {
                                        a("/person/${u.id}") {
                                            i("material-icons") { +"edit" }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    get<LocationPerson.Edit> { personId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.PERSON)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val editPerson = Person.get(personId.id) ?: return@get
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.PERSON
                }
                content {
                    h1 { +"Edit person" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "first-name"
                                +"First name"
                            }
                            input(
                                name = "first-name",
                                classes = "form-control"
                            ) {
                                id = "first-name"
                                placeholder = "First name"
                                value = editPerson.firstName
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "last-name"
                                +"Last name"
                            }
                            input(
                                name = "last-name",
                                classes = "form-control"
                            ) {
                                id = "last-name"
                                placeholder = "Last name"
                                value = editPerson.lastName
                            }
                        }

                        div("form-group") {
                            a("/person") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Save"
                            }
                        }
                    }
                    a("/person/${editPerson.id}/delete") {
                        button(classes = "form-btn btn-danger") {
                            +"Delete"
                        }
                    }
                }
            }
        }
    }

    post<LocationPerson.Edit> { personId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.PERSON)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }
            val editPerson = Person.get(personId.id) ?: return@post

            params["first-name"]?.let { editPerson.firstName = it }
            params["last-name"]?.let { editPerson.lastName = it }

            editPerson.save()

            call.respondRedirect("/person")
        }
    }

    get<LocationPerson.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.PERSON)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.PERSON
                }
                content {
                    h1 { +"Create person" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "first-name"
                                +"First name"
                            }
                            input(
                                name = "first-name",
                                classes = "form-control"
                            ) {
                                id = "first-name"
                                placeholder = "First name"
                                value = ""
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "last-name"
                                +"Last name"
                            }
                            input(
                                name = "last-name",
                                classes = "form-control"
                            ) {
                                id = "last-name"
                                placeholder = "Last name"
                                value = ""
                            }
                        }

                        div("form-group") {
                            a("/person") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Create"
                            }
                        }
                    }
                }
            }
        }
    }

    post<LocationPerson.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.PERSON)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }

            val firstName = params["first-name"] ?: return@post
            val lastName = params["last-name"] ?: return@post

            Person(firstName = firstName, lastName = lastName).save()

            call.respondRedirect("/person")
        }
    }

    get<LocationPerson.Delete> { personId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.PERSON)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val deletePerson = Person.get(personId.id) ?: return@get

            deletePerson.delete()

            call.respondRedirect("/person")
        }
    }
}
