package de.kif.backend.route

import de.kif.backend.LocationLogin
import de.kif.backend.LocationRoom
import de.kif.backend.PortalSession
import de.kif.backend.model.Permission
import de.kif.backend.model.Room
import de.kif.backend.util.Search
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.backend.view.TableTemplate
import io.ktor.application.call
import io.ktor.html.insert
import io.ktor.html.respondHtmlTemplate
import io.ktor.locations.get
import io.ktor.locations.post
import io.ktor.request.path
import io.ktor.request.receiveParameters
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.sessions.get
import io.ktor.sessions.sessions
import io.ktor.util.toMap
import kotlinx.html.*

fun Route.room() {
    get<LocationRoom> { param ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.ROOM)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val list = Room.list()
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.ROOM
                }
                content {
                    h1 { +"Rooms" }
                    insert(TableTemplate()) {
                        searchValue = param.search

                        action {
                            a("/room/new") {
                                button(classes="form-btn btn-primary") {
                                    +"Add room"
                                }
                            }
                        }

                        header {
                            th {
                                +"Name"
                            }
                            th {
                                +"Places"
                            }
                            th {
                                +"Projector"
                            }
                            th(classes = "action") {
                                +"Action"
                            }
                        }

                        for (u in list) {
                            if (Search.match(param.search, u.name)) {
                                entry {
                                    attributes["data-search"] = Search.pack(u.name)
                                    td {
                                        +u.name
                                    }
                                    td {
                                        +u.places.toString()
                                    }
                                    td {
                                        +u.projector.toString()
                                    }
                                    td(classes = "action") {
                                        a("/room/${u.id}") {
                                            i("material-icons") { +"edit" }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    get<LocationRoom.Edit> { roomId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.ROOM)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val editRoom = Room.get(roomId.id)
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.ROOM
                }
                content {
                    h1 { +"Edit room" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "name"
                                +"Name"
                            }
                            input(
                                name = "name",
                                classes = "form-control"
                            ) {
                                id = "name"
                                placeholder = "Name"
                                value = editRoom.name
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "places"
                                +"Places"
                            }
                            input(
                                name = "places",
                                classes = "form-control",
                                type = InputType.number
                            ) {
                                id = "places"
                                placeholder = "Places"
                                value = editRoom.places.toString()

                                min = "0"
                                max = "1337"
                            }
                        }

                        div("form-switch-group") {
                            div("form-group form-switch") {
                                input(
                                    name = "projector",
                                    classes = "form-control",
                                    type = InputType.checkBox
                                ) {
                                    id = "projector"
                                    checked = editRoom.projector
                                }
                                label {
                                    htmlFor = "projector"
                                    +"Projector"
                                }
                            }
                        }

                        div("form-group") {
                            a("/room") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Save"
                            }
                        }
                    }
                    a("/room/${editRoom.id}/delete") {
                        button(classes = "form-btn btn-danger") {
                            +"Delete"
                        }
                    }
                }
            }
        }
    }

    post<LocationRoom.Edit> { roomId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.ROOM)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }
            val editRoom = Room.get(roomId.id)

            params["name"]?.let { editRoom.name = it }
            params["places"]?.let { editRoom.places = it.toIntOrNull() ?: 0 }
            params["projector"]?.let { editRoom.projector = it == "on" }

            editRoom.save()

            call.respondRedirect("/room")
        }
    }

    get<LocationRoom.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.ROOM)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.ROOM
                }
                content {
                    h1 { +"Create room" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "name"
                                +"Name"
                            }
                            input(
                                name = "name",
                                classes = "form-control"
                            ) {
                                id = "name"
                                placeholder = "Name"
                                value = ""
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "places"
                                +"Places"
                            }
                            input(
                                name = "places",
                                classes = "form-control",
                                type = InputType.number
                            ) {
                                id = "places"
                                placeholder = "Places"
                                value = "0"

                                min = "0"
                                max = "1337"
                            }
                        }

                        div("form-switch-group") {
                            div("form-group form-switch") {
                                input(
                                    name = "projector",
                                    classes = "form-control",
                                    type = InputType.checkBox
                                ) {
                                    id = "projector"
                                    checked = false
                                }
                                label {
                                    htmlFor = "projector"
                                    +"Projector"
                                }
                            }
                        }

                        div("form-group") {
                            a("/room") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Create"
                            }
                        }
                    }
                }
            }
        }
    }

    post<LocationRoom.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.ROOM)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }

            val name = params["name"] ?: return@post
            val places = (params["places"] ?: return@post).toIntOrNull() ?: 0
            val projector = params["projector"] == "on"

            Room(name = name, places = places, projector = projector).save()

            call.respondRedirect("/room")
        }
    }

    get<LocationRoom.Delete> { roomId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.ROOM)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val deleteRoom = Room.get(roomId.id)

            deleteRoom.delete()

            call.respondRedirect("/room")
        }
    }
}
