package de.kif.backend.route

import de.kif.backend.LocationLogin
import de.kif.backend.LocationTrack
import de.kif.backend.PortalSession
import de.kif.backend.model.Permission
import de.kif.backend.model.Track
import de.kif.backend.util.Search
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.backend.view.TableTemplate
import io.ktor.application.call
import io.ktor.html.insert
import io.ktor.html.respondHtmlTemplate
import io.ktor.locations.get
import io.ktor.locations.post
import io.ktor.request.path
import io.ktor.request.receiveParameters
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.sessions.get
import io.ktor.sessions.sessions
import io.ktor.util.toMap
import kif.common.model.Color
import kotlinx.css.CSSBuilder
import kotlinx.html.*
import kotlin.random.Random

fun DIV.colorPicker(color: Color?) {
    val colorString = color?.toString() ?: Color(
        Random.nextInt(256),
        Random.nextInt(256),
        Random.nextInt(256)
    ).toString()

    var found = false
    fun DIV.generate(colorId: String, c: Color) {
        val name = colorId.replace("-", " ").capitalize()
        div("color-picker-entry") {
            radioInput(name = "color") {
                id = "color-$colorId"
                value = colorId

                if (color == c) {
                    checked = true
                    found = true
                }
            }
            label {
                title = name
                htmlFor = "color-$colorId"
                attributes["style"] = CSSBuilder().apply {
                    backgroundColor = kotlinx.css.Color(c.toString())
                }.toString()
            }
        }
    }
    div("color-picker") {
        for ((name, c) in Color.default) {
            generate(name, c)
        }

        div("color-picker-custom") {
            radioInput(name = "color") {
                id = "color-custom"
                value = "custom"

                if (!found) {
                    checked = true
                }
            }
            label {
                title = "Custom"
                htmlFor = "color-custom"

                input(
                    name = "color-input",
                    type = InputType.color
                ) {
                    value = colorString
                }
            }
        }
    }
}

fun Route.track() {
    get<LocationTrack> { param ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val list = Track.list()
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.WORK_GROUP
                }
                content {
                    h1 { +"Tracks" }
                    insert(TableTemplate()) {
                        searchValue = param.search

                        action {
                            a("/track/new") {
                                button(classes = "form-btn btn-primary") {
                                    +"Add track"
                                }
                            }
                        }

                        header {
                            th {
                                +"Name"
                            }
                            th {
                                +"Color"
                            }
                            th(classes = "action") {
                                +"Action"
                            }
                        }

                        for (u in list) {
                            if (Search.match(param.search, u.name)) {
                                entry {
                                    attributes["data-search"] = Search.pack(u.name)
                                    td {
                                        +u.name
                                    }
                                    td {
                                        +u.color.toString()
                                    }
                                    td(classes = "action") {
                                        a("/track/${u.id}") {
                                            i("material-icons") { +"edit" }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    get<LocationTrack.Edit> { trackId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val editTrack = Track.get(trackId.id)
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.WORK_GROUP
                }
                content {
                    h1 { +"Edit track" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "name"
                                +"Name"
                            }
                            input(
                                name = "name",
                                classes = "form-control"
                            ) {
                                id = "name"
                                placeholder = "Name"
                                value = editTrack.name
                            }
                        }
                        div("form-group") {
                            colorPicker(editTrack.color)
                        }

                        div("form-group") {
                            a("/track") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Save"
                            }
                        }
                    }
                    a("/track/${editTrack.id}/delete") {
                        button(classes = "form-btn btn-danger") {
                            +"Delete"
                        }
                    }
                }
            }
        }
    }

    post<LocationTrack.Edit> { trackId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }
            val editTrack = Track.get(trackId.id)

            params["name"]?.let { editTrack.name = it }

            editTrack.color = (params["color"] ?: return@post).let { c ->
                Color.default.find { it.first == c }
            }?.second ?: (params["color-input"] ?: return@post).let(Color.Companion::parse)

            editTrack.save()

            call.respondRedirect("/track")
        }
    }

    get<LocationTrack.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.WORK_GROUP
                }
                content {
                    h1 { +"Create track" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "name"
                                +"Name"
                            }
                            input(
                                name = "name",
                                classes = "form-control"
                            ) {
                                id = "name"
                                placeholder = "Name"
                                value = ""
                            }
                        }
                        div("form-group") {
                            colorPicker(null)
                        }
                        div("form-group") {
                            a("/track") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Create"
                            }
                        }
                    }
                }
            }
        }
    }

    post<LocationTrack.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }

            val name = params["name"] ?: return@post
            val color = (params["color"] ?: return@post).let { c ->
                Color.default.find { it.first == c }
            }?.second ?: (params["color-input"] ?: return@post).let(Color.Companion::parse)

            Track(
                name = name,
                color = color
            ).save()

            call.respondRedirect("/track")
        }
    }

    get<LocationTrack.Delete> { trackId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val deleteTrack = Track.get(trackId.id)

            deleteTrack.delete()

            call.respondRedirect("/track")
        }
    }
}
