package de.kif.backend.route

import de.kif.backend.LocationLogin
import de.kif.backend.LocationUser
import de.kif.backend.PortalSession
import de.kif.backend.model.Permission
import de.kif.backend.model.User
import de.kif.backend.util.Search
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.backend.view.TableTemplate
import io.ktor.application.call
import io.ktor.html.insert
import io.ktor.html.respondHtmlTemplate
import io.ktor.locations.get
import io.ktor.locations.post
import io.ktor.request.path
import io.ktor.request.receiveParameters
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.sessions.get
import io.ktor.sessions.sessions
import io.ktor.util.toMap
import kotlinx.html.*

fun Route.user() {
    get<LocationUser> { param ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.USER)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val list = User.list()
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.USER
                }
                content {
                    h1 { +"Users" }
                    insert(TableTemplate()) {
                        searchValue = param.search

                        action {
                            a("/user/new") {
                                button(classes="form-btn btn-primary") {
                                    +"Add user"
                                }
                            }
                        }

                        header {
                            th {
                                +"Username"
                            }
                            th {
                                +"Permissions"
                            }
                            th(classes = "action") {
                                +"Action"
                            }
                        }

                        for (u in list) {
                            if (Search.match(param.search, u.username)) {
                                entry {
                                    attributes["data-search"] = Search.pack(u.username)
                                    td {
                                        +u.username
                                    }
                                    td {
                                        +u.permissions.joinToString(", ") { it.toString().toLowerCase() }
                                    }
                                    td(classes = "action") {
                                        a("/user/${u.id}") {
                                            i("material-icons") { +"edit" }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    get<LocationUser.Edit> { userId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.USER)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val editUser = User.get(userId.id) ?: return@get
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.USER
                }
                content {
                    h1 { +"Edit user" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "username"
                                +"Username"
                            }
                            input(
                                name = "username",
                                classes = "form-control"
                            ) {
                                id = "username"
                                placeholder = "Username"
                                value = editUser.username
                            }
                        }

                        div("form-switch-group") {
                            for (permission in Permission.values()) {
                                val name = permission.toString().toLowerCase()
                                div("form-group form-switch") {
                                    input(
                                        name = "permission-$name",
                                        classes = "form-control",
                                        type = InputType.checkBox
                                    ) {
                                        id = "permission-$name"
                                        checked = permission in editUser.permissions

                                        if (!user.checkPermission(permission)) {
                                            readonly = true
                                        }
                                    }
                                    label {
                                        htmlFor = "permission-$name"
                                        +name.capitalize()
                                    }
                                }
                            }
                        }

                        div("form-group") {
                            a("/user") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Save"
                            }
                        }
                    }
                    a("/user/${editUser.id}/delete") {
                        button(classes = "form-btn btn-danger") {
                            +"Delete"
                        }
                    }
                }
            }
        }
    }

    post<LocationUser.Edit> { userId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.USER)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }
            val editUser = User.get(userId.id) ?: return@post

            params["username"]?.let { editUser.username = it }

            for (permission in Permission.values()) {
                val name = permission.toString().toLowerCase()
                if (user.checkPermission(permission)) {
                    if (params["permission-$name"] == "on") {
                        editUser.permissions += permission
                    } else {
                        editUser.permissions -= permission
                    }
                }
            }
            editUser.save()

            call.respondRedirect("/user")
        }
    }

    get<LocationUser.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.USER)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.USER
                }
                content {
                    h1 { +"Create user" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "username"
                                +"Username"
                            }
                            input(
                                name = "username",
                                classes = "form-control"
                            ) {
                                id = "username"
                                placeholder = "Username"
                                value = ""
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "password"
                                +"Password"
                            }
                            input(
                                name = "password",
                                classes = "form-control",
                                type = InputType.password
                            ) {
                                id = "password"
                                placeholder = "Password"
                                value = ""
                            }
                        }

                        div("form-switch-group") {
                            for (permission in Permission.values()) {
                                val name = permission.toString().toLowerCase()
                                div("form-group form-switch") {
                                    input(
                                        name = "permission-$name",
                                        classes = "form-control",
                                        type = InputType.checkBox
                                    ) {
                                        id = "permission-$name"
                                        checked = false

                                        if (!user.checkPermission(permission)) {
                                            readonly = true
                                        }
                                    }
                                    label {
                                        htmlFor = "permission-$name"
                                        +name.capitalize()
                                    }
                                }
                            }
                        }

                        div("form-group") {
                            a("/user") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Create"
                            }
                        }
                    }
                }
            }
        }
    }

    post<LocationUser.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.USER)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }

            val username = params["username"] ?: return@post
            val password = params["password"] ?: return@post

            val permissions = Permission.values().mapNotNull { permission ->
                if (user.checkPermission(permission) && params["permission-${permission.toString().toLowerCase()}"] == "on")
                    permission
                else
                    null
            }.toSet()

            User.create(username, password, permissions)

            call.respondRedirect("/user")
        }
    }

    get<LocationUser.Delete> { userId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.USER)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val deleteUser = User.get(userId.id) ?: return@get

            if (user.checkPermission(Permission.USER) &&
                (Permission.ADMIN !in deleteUser.permissions || Permission.ADMIN in user.permissions)
            ) {
                deleteUser.delete()
            }

            call.respondRedirect("/user")
        }
    }
}
