package de.kif.backend.route

import de.kif.backend.LocationLogin
import de.kif.backend.LocationWorkGroup
import de.kif.backend.PortalSession
import de.kif.backend.database.Language
import de.kif.backend.model.Permission
import de.kif.backend.model.Track
import de.kif.backend.model.WorkGroup
import de.kif.backend.util.Search
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import de.kif.backend.view.TableTemplate
import io.ktor.application.call
import io.ktor.html.insert
import io.ktor.html.respondHtmlTemplate
import io.ktor.locations.get
import io.ktor.locations.post
import io.ktor.request.path
import io.ktor.request.receiveParameters
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.sessions.get
import io.ktor.sessions.sessions
import io.ktor.util.toMap
import kotlinx.html.*

fun Route.workGroup() {
    get<LocationWorkGroup> { param ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val list = WorkGroup.list()
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.WORK_GROUP
                }
                content {
                    h1 { +"Work groups" }
                    insert(TableTemplate()) {
                        searchValue = param.search

                        action {
                            a("/track") {
                                button(classes = "form-btn") {
                                    +"Edit tracks"
                                }
                            }
                            a("/workgroup/new") {
                                button(classes = "form-btn btn-primary") {
                                    +"Add work group"
                                }
                            }
                        }

                        header {
                            th {
                                +"Name"
                            }
                            th {
                                +"Interested"
                            }
                            th {
                                +"Track"
                            }
                            th {
                                +"Projector"
                            }
                            th {
                                +"Resolution"
                            }
                            th {
                                +"Length"
                            }
                            th {
                                +"Language"
                            }
                            th(classes = "action") {
                                +"Action"
                            }
                        }

                        for (u in list) {
                            if (Search.match(param.search, u.name)) {
                                entry {
                                    attributes["data-search"] = Search.pack(u.name)
                                    td {
                                        +u.name
                                    }
                                    td {
                                        +u.interested.toString()
                                    }
                                    td {
                                        +(u.track?.name ?: "")
                                    }
                                    td {
                                        +u.projector.toString()
                                    }
                                    td {
                                        +u.resolution.toString()
                                    }
                                    td {
                                        +u.length.toString()
                                    }
                                    td {
                                        +u.language.toString()
                                    }
                                    td(classes = "action") {
                                        a("/workgroup/${u.id}") {
                                            i("material-icons") { +"edit" }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    get<LocationWorkGroup.Edit> { workGroupId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val editWorkGroup = WorkGroup.get(workGroupId.id)
            val tracks = Track.list()
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.WORK_GROUP
                }
                content {
                    h1 { +"Edit work group" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "name"
                                +"Name"
                            }
                            input(
                                name = "name",
                                classes = "form-control"
                            ) {
                                id = "name"
                                placeholder = "Name"
                                value = editWorkGroup.name
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "interested"
                                +"Interested"
                            }
                            input(
                                name = "interested",
                                classes = "form-control",
                                type = InputType.number
                            ) {
                                id = "interested"
                                value = editWorkGroup.interested.toString()

                                min = "0"
                                max = "1337"
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "track"
                                +"Track"
                            }
                            div("input-group") {
                                select(
                                    classes = "form-control"
                                ) {
                                    name = "track"

                                    option {
                                        selected = (editWorkGroup.trackId ?: -1) < 0
                                        value = "-1"
                                        +"None"
                                    }
                                    for (track in tracks) {
                                        option {
                                            selected = editWorkGroup.trackId == track.id
                                            value = track.id.toString()
                                            +track.name
                                        }
                                    }
                                }

                                a("/track", classes = "form-btn") {
                                    i("material-icons") { +"edit" }
                                }
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "length"
                                +"Length"
                            }
                            input(
                                name = "length",
                                classes = "form-control",
                                type = InputType.number
                            ) {
                                id = "length"
                                value = editWorkGroup.length.toString()

                                min = "0"
                                max = "1440"
                                step = "5"
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "language"
                                +"Language"
                            }
                            select(
                                classes = "form-control"
                            ) {
                                name = "language"

                                for (language in Language.values()) {
                                    option {
                                        selected = editWorkGroup.language == language
                                        value = language.name
                                        +language.toString()
                                    }
                                }
                            }
                        }

                        div("form-switch-group") {
                            div("form-group form-switch") {
                                input(
                                    name = "projector",
                                    classes = "form-control",
                                    type = InputType.checkBox
                                ) {
                                    id = "projector"
                                    checked = editWorkGroup.projector
                                }
                                label {
                                    htmlFor = "projector"
                                    +"Projector"
                                }
                            }
                            div("form-group form-switch") {
                                input(
                                    name = "resolution",
                                    classes = "form-control",
                                    type = InputType.checkBox
                                ) {
                                    id = "resolution"
                                    checked = editWorkGroup.resolution
                                }
                                label {
                                    htmlFor = "resolution"
                                    +"Resolution"
                                }
                            }
                        }

                        div("form-group") {
                            a("/workgroup") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Save"
                            }
                        }
                    }
                    a("/workgroup/${editWorkGroup.id}/delete") {
                        button(classes = "form-btn btn-danger") {
                            +"Delete"
                        }
                    }
                }
            }
        }
    }

    post<LocationWorkGroup.Edit> { workGroupId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }
            val editWorkGroup = WorkGroup.get(workGroupId.id)

            params["name"]?.let { editWorkGroup.name = it }
            params["interested"]?.toIntOrNull()?.let { editWorkGroup.interested = it }
            params["track"]?.toIntOrNull()?.let { editWorkGroup.trackId = it }
            params["projector"]?.let { editWorkGroup.projector = it == "on" }
            params["resolution"]?.let { editWorkGroup.resolution = it == "on" }
            params["length"]?.toIntOrNull()?.let { editWorkGroup.length = it }
            params["language"]?.let { editWorkGroup.language = Language.values().find { l -> l.name == it } ?: Language.GERMAN }

            editWorkGroup.save()

            call.respondRedirect("/workgroup")
        }
    }

    get<LocationWorkGroup.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val tracks = Track.list()
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.WORK_GROUP
                }
                content {
                    h1 { +"Create work group" }
                    form(method = FormMethod.post) {
                        div("form-group") {
                            label {
                                htmlFor = "name"
                                +"Name"
                            }
                            input(
                                name = "name",
                                classes = "form-control"
                            ) {
                                id = "name"
                                placeholder = "Name"
                                value = ""
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "interested"
                                +"Interested"
                            }
                            input(
                                name = "interested",
                                classes = "form-control",
                                type = InputType.number
                            ) {
                                id = "interested"
                                value = "0"

                                min = "0"
                                max = "1337"
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "track"
                                +"Track"
                            }
                            select(
                                classes = "form-control"
                            ) {
                                name = "track"

                                option {
                                    selected = true
                                    value = "-1"
                                    +"None"
                                }
                                for (track in tracks) {
                                    option {
                                        value = track.id.toString()
                                        +track.name
                                    }
                                }
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "length"
                                +"Length"
                            }
                            input(
                                name = "length",
                                classes = "form-control",
                                type = InputType.number
                            ) {
                                id = "length"
                                value = "0"

                                min = "0"
                                max = "1440"
                                step = "5"
                            }
                        }
                        div("form-group") {
                            label {
                                htmlFor = "language"
                                +"Language"
                            }
                            select(
                                classes = "form-control"
                            ) {
                                name = "language"

                                for (language in Language.values()) {
                                    option {
                                        selected = language == Language.GERMAN
                                        value = language.name
                                        +language.toString()
                                    }
                                }
                            }
                        }

                        div("form-switch-group") {
                            div("form-group form-switch") {
                                input(
                                    name = "projector",
                                    classes = "form-control",
                                    type = InputType.checkBox
                                ) {
                                    id = "projector"
                                    checked = false
                                }
                                label {
                                    htmlFor = "projector"
                                    +"Projector"
                                }
                            }
                            div("form-group form-switch") {
                                input(
                                    name = "resolution",
                                    classes = "form-control",
                                    type = InputType.checkBox
                                ) {
                                    id = "resolution"
                                    checked = false
                                }
                                label {
                                    htmlFor = "resolution"
                                    +"Resolution"
                                }
                            }
                        }

                        div("form-group") {
                            a("/workgroup") {
                                button(classes = "form-btn") {
                                    +"Cancel"
                                }
                            }
                            button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                +"Create"
                            }
                        }
                    }
                }
            }
        }
    }

    post<LocationWorkGroup.New> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val params = call.receiveParameters().toMap().mapValues { (_, list) ->
                list.firstOrNull()
            }

            val name = params["name"] ?: return@post
            val interested = (params["interested"] ?: return@post).toIntOrNull() ?: 0
            val trackId = (params["track"] ?: return@post).toIntOrNull()
            val projector = params["projector"] == "on"
            val resolution = params["resolution"] == "on"
            val length = (params["length"] ?: return@post).toIntOrNull() ?: 0
            val language = (params["language"] ?: return@post).let { Language.values().find { l -> l.name == it } ?: Language.GERMAN }

            WorkGroup(
                name = name,
                interested = interested,
                trackId = trackId,
                projector = projector,
                resolution = resolution,
                length = length,
                language = language
            ).save()

            call.respondRedirect("/workgroup")
        }
    }

    get<LocationWorkGroup.Delete> { workGroupId ->
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null || !user.checkPermission(Permission.WORK_GROUP)) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            val deleteWorkGroup = WorkGroup.get(workGroupId.id)

            deleteWorkGroup.delete()

            call.respondRedirect("/workgroup")
        }
    }
}
